(* In this unit are all global variables inside and Memory functions inside *)
(* and also every misc stuff!                                               *)
{$i compile.inc}
Unit vars {& Misc};

interface

type ZXModels=(mZX48,mZX128,mZXPlus2,mZXPlus3,mPentagon,mScorpion);
const
           {Display/ULA/Memory constants}
           Max_V_line         : Word = 311;
           t_states_per_line  : Word = 228;
           t_state            : Word = 0;
           ula_delay          : Word = 0;
           cpu_delay          : Word = 0;
           frame_skip         : Byte = 1;
           Model              : ZXModels=mPentagon;
           {Display}
           vline:Word=0;
           flash_counter:Byte=0;
           flash_status:Boolean=False;

           {Others}
           usestick: Boolean=False;

           {Timing}
           timerVar:word=0;
           timerproc: pointer=NIL;
           RealSpeed: Boolean=True;

{Registers...................................................................}

type
     pair=record
        case boolean of
           false: (l,h:Byte);
           true: (W:Word);
        end;
        quadruple=record
           case byte of
              0: (b1,b2,b3,b4:Byte);
              1: (w1,w2:Word);
              2: (q:longint);
           end;

var af, bc, de, hl:pair;
    ix, iy:pair;
    sp, pc:pair;
    ir: Pair;
    im   : Byte;
    AF2,BC2,DE2,HL2: Pair; { == AF' BC' DE' HL'}
    iff1, iff2, bit7_r: Byte;
    border:Byte;

    halt, quit:Byte;
    qtemp: quadruple;
    ptemp:pair;

{Sound.......................................................................}

    PSG:array[0..15] of byte; {AY 8910}
    last_fffd:Byte;           {AY 8910}

{Keyboard....................................................................}

    skey:Array[0..7] of byte; {Each Spectrum Key is saved inside this Array}
    stickvalue:Byte; {Joystick}
    joykeys: record up,down,left,right,fire:byte; end;

{Memory......................................................................}

type tmem=array[0..$4000] of byte;
     pmem=^tmem;
     tmemory=array[0..$C000-1{64000}] of byte;

var  ram_state: Byte;
     contended: array[0..7]of byte;

const
     RAM          : array[0..7] of pmem=(NIL,NIL,NIL,NIL,NIL,NIL,NIL,NIL);
     SRAM         : array[0..3] of pmem=(NIL,NIL,NIL,NIL);

     ZX48_ROM     : pmem=NIL;
     TRDOSROM     : pmem=NIL;
     SP_SCREEN    : pmem=NIL;

     ZX128_ROM    : array[0..1] of pmem=(NIL,NIL);
     PLUS2_ROM    : array[0..1] of pmem=(NIL,NIL);
     PLUS3_ROM    : array[0..3] of pmem=(NIL,NIL,NIL,NIL);

     PENTAGON_ROM : array[0..1] of pmem=(NIL,NIL);
     SCORPION_ROM : array[0..3] of pmem=(NIL,NIL,NIL,NIL);

     BUFFER:^tmemory=NIL;

function speekb(Addr:Word):Byte;
function wordpeek(Addr:Word):Word;
procedure spokeb(addr:Word;val:Byte);
procedure wordpoke(addr,val:Word);
Procedure InitMem;
Procedure CloseMem;

{$i tables.inc}

implementation

uses CRT,DOS,Core;

function speekb(Addr:Word):Byte;
begin
 speekb:=SRAM[(addr and $C000) shr 14]^[addr and $3FFF]
end;

function wordpeek(Addr:Word):Word;
begin
 wordpeek:=((speekb((addr)+1) shl 8) or speekb(addr))
end;

procedure spokeb(addr:word;val:Byte);
var segm:Word;
begin
 segm:=(addr and $C000) shr 14;
 if segm>=1 then SRAM[segm]^[addr and $3FFF]:=val;
 if (CONTENDED[segm]>0) and
    (t_state shr 7=0) and (vline and 192>0) then inc(t_state,4);
end;

procedure wordpoke(addr,val:Word);
begin
 spokeb(addr, LO(val));spokeb(addr + 1, HI(val));
end;

procedure AllocFree(var ROM:Pmem;S:String;x:byte);
var xs:string;
begin
  if x=0 then xs:='' else xs:='['+i2s(x,1,' ')+']';
  halt_error:='cannot deallocate memory for Spectrum '+s+xs;

 if ROM<>NIL then freemem(Rom,$4000);
end;

procedure MemAlloc(var ROM:pmem;S:String;x:Byte);
var xs:string;
begin
 Getmem(Rom,$4000);
 if ROM=NIL then
 begin
  if x=0 then xs:='' else xs:='['+i2s(x,1,' ')+']';
  halt_error:='cannot allocate memory for Spectrum '+s+xs;
  system.halt;
 end;
end;

procedure initmem;
var x: byte;
    s:String;
begin
        MemAlloc(trdosrom,'TRDOS ',0);
        MemAlloc(ZX48_ROM,'ZX48 ',0);

        for x:=0 to 7 do MemAlloc(ram[x],'RAM ',x);

        SRAM[1]:=RAM[5];
        SRAM[2]:=RAM[2];
        SRAM[3]:=RAM[0];

        for x:=0 to 1 do MemAlloc(ZX128_ROM[x],'ZX128 ROM ',x);
        for x:=0 to 1 do MemAlloc(Plus2_ROM[x],'PLUS2 ROM ',x);
        for x:=0 to 3 do MemAlloc(Plus3_ROM[x],'PLUS3 ROM ',x);
        for x:=0 to 1 do Memalloc(PENTAGON_ROM[x],'PENTAGON ',x);
        for x:=0 to 3 do Memalloc(SCORPION_ROM[x],'SCORPION ',x);


        SRAM[0]:=ZX128_ROM[1];
        SP_SCREEN:=RAM[5];

        halt_Error:='cannot allocate memory for temporary buffer';
        getmem(buffer,sizeof(tmemory));

        CONTENDED[0]:=0;CONTENDED[1]:=0;
        CONTENDED[2]:=1;CONTENDED[3]:=1;
        CONTENDED[4]:=0;CONTENDED[5]:=0;
        CONTENDED[6]:=0;CONTENDED[7]:=0;

end;

procedure closemem;
var x: byte;
begin
 allocfree(trdosrom,'TRDOS ROM',0);
 allocfree(zx48_ROM,'ZX48k ROM',0);

 for x:=0 to 7 do allocfree(ram[x],'RAM ',x);
 for x:=0 to 1 do allocfree(ZX128_ROM[x],'ZX128 ROM',x);
 for x:=0 to 1 do allocfree(Plus2_ROM[x],'PLUS2 ROM',x);
 for x:=0 to 3 do allocfree(Plus3_ROM[x],'PLUS3 ROM',x);
 for x:=0 to 1 do allocfree(PENTAGON_ROM[x],'PENTAGON ROM',x);
 for x:=0 to 3 do allocfree(SCORPION_ROM[x],'SCORPION ROM',x);

 halt_Error:='cannot deallocate memory for temporary buffer';
 freemem(buffer,sizeof(tmemory));
 halt_error:='';
end;


procedure calc_intable;
var i,j:Byte;
begin
  for i:=0 to 255 do
  begin
        if i=0 then j:=64 else j:=0;
        add8_table[i]:=(i and 168) or (j); (* Just HVC to do! *)
        sub8_table[i]:=(i and 168) or (j) or 2;
        cpsub8_table[i]:=(i and 128) or (j) or 2; (* 5 bits left.. *)
        in_table[i]:=(i and 168) or (parity[i]);
        in_table[0]:=in_table[0] or 64; (* Don't forget to keep old C flag *)
  end;
end;
var s:String;
BEGIN
 textattr:=7;ClrScr;
 textcolor(15); TextBackground(4);
 s:='';while length(s)<80 do s:=s+' ';
 gotoxy(1,1);writeln(s);
 WriteCentered('Initialising System',1);
 Textcolor(8); Textbackground(0);
 gotoxy(1,2);
 Window(2,3,78,21);
 gotoxy(1,1);
 Textcolor(7); TextBackground(0);

 InitNew('Variables & Memory ');

 calc_intable;
end.
